#define F_CPU 8000000UL

#define TRIG_DDR  DDRD
#define TRIG_PORT PORTD
#define TRIG_PIN  PIND3

#include <avr/io.h>
#include <util/delay.h>
#include <avr/interrupt.h>
#include <stdbool.h>
#include "hcsr04.h"

uint16_t rising, falling;
uint32_t counts;
int dist;

bool star = false;

//   
ISR(TIMER1_CAPT_vect) {
  //  ICES1 = 1
  //   -  
  if (TCCR1B & (1<<ICES1)) {
    //     
	rising = ICR1;
    //       ( )
    TCCR1B &= ~(1<<ICES1);
    //    ,    
    dist = -2;
  } 
  //  ICES1 = 0
  //   -  
  else {
    //     
    falling = ICR1;
    //    
    counts = falling - rising;
    //     
    dist = (counts * 10) / 58;
    //    waitHCSR04()   
    //    ..   
    star = false;
  }
}

//           OCR1A
//  OCR1A = 0xFFFF
ISR(TIMER1_COMPA_vect) {
  //    waitHCSR04()   
  //    ..   
  //  HC-SR04     .. 
  //   ,       
  dist = -1;
  star = false;
}

//   HCSR04
inline void initHCSR04() {
  //    TRIG  
  TRIG_DDR |= 1<<TRIG_PIN;
  rising = falling = counts = dist = 0;
  //    1 TIMER1
  //  CS12,CS11,CS10: Clock Select.  3       .
  //    0    1    0 : F_CPU/8 (  )
  //  ICNC1:     . 1    
  //  WGM13,WGM12,WGM11,WGM10:  
  //    0     1     0     0  :  CTC,   OCR1A = 0xFFFF
  TCCR1B |= (1<<ICNC1) | (1<<CS11) | (1<<WGM12); 
  //  ICIE1 = 1:    / 1
  //  OCIE1A = 1:      A   /1
  TIMSK |= (1<<ICIE1) | (1<<OCIE1A); 
  //  ICES1 = 1:   ICR1    ( ).
  TCCR1B |= (1<<ICES1);
  //    
  OCR1A = 0xFFFF;
  //   
  sei();
}

//    
inline void stopHCSR04() {
  TRIG_DDR &= ~(1<<TRIG_PIN);
  // TIMER1 INIT
  TCCR1B &= ((1<<ICNC1) | (1<<CS11) | (1<<WGM12));
  TIMSK &= ((1<<ICIE1) | (1<<OCIE1A));
  TCCR1B &= ((1<<ICES1));
  OCR1A = 0;
}

//     12us
inline void trigHCSR04() {
  TRIG_PORT ^= (1<<TRIG_PIN);
  _delay_us(12);
  TRIG_PORT ^= (1<<TRIG_PIN);
}

//    star  true ..  
inline void waitHCSR04() {
  star = true;
  while(star) {
    _delay_us(1);
  }
}

int getDistance() {
  //    (HCSR04)
  initHCSR04();
  //     12us
  trigHCSR04();
  //  ,    
  waitHCSR04();
  //   
  stopHCSR04();
  //   
  return dist;
}